@@ -30,7 +30,7 @@ from .command import (build, build_py, clean,
                       bdist, bdist_wheel, egg_info,
                       sdist, generate_source_manifest)
 from .constants import CMAKE_INSTALL_DIR
-from .exceptions import SKBuildError
+from .exceptions import SKBuildError, SKBuildGeneratorNotFoundError
 from .utils import (mkdir_p, PythonModuleFinder, to_platform_path, to_unix_path)
 
 
@@ -116,7 +116,9 @@ def _parse_setuptools_arguments(setup_attrs):
     """This function instantiates a Distribution object and
     parses the command line arguments.
 
-    It returns a tuple (display_only, help_commands, commands, hide_listing)
+    It returns the tuple
+        ``(display_only, help_commands, commands,
+        hide_listing, force_cmake, skip_cmake)``
     where
     - display_only is a boolean indicating if an argument like '--help',
       '--help-commands' or '--author' was passed.
@@ -125,6 +127,9 @@ def _parse_setuptools_arguments(setup_attrs):
     - commands contains the list of commands that were passed.
     - hide_listing is a boolean indicating if the list of files being included
       in the distribution is displayed or not.
+    - force_cmake a boolean indicating that CMake should always be executed.
+    - skip_cmake is a boolean indicating if the execution of CMake should
+      explicitly be skipped.
 
     Otherwise it raises DistutilsArgError exception if there are
     any error on the command-line, and it raises DistutilsGetoptError
@@ -141,10 +146,19 @@ def _parse_setuptools_arguments(setup_attrs):
 
     # Update class attribute to also ensure the argument is processed
     # when ``upstream_setup`` is called.
+    # pylint:disable=no-member
     upstream_Distribution.global_options.append(
         ('hide-listing', None, "do not display list of files being "
                                "included in the distribution")
     )
+    # pylint:disable=no-member
+    upstream_Distribution.global_options.append(
+        ('force-cmake', None, "always run CMake")
+    )
+    # pylint:disable=no-member
+    upstream_Distribution.global_options.append(
+        ('skip-cmake', None, "do not run CMake")
+    )
 
     # Find and parse the config file(s): they will override options from
     # the setup script, but be overridden by the command line.
@@ -159,8 +173,13 @@ def _parse_setuptools_arguments(setup_attrs):
         display_only = not result
         if not hasattr(dist, 'hide_listing'):
             dist.hide_listing = False
+        if not hasattr(dist, 'force_cmake'):
+            dist.force_cmake = False
+        if not hasattr(dist, 'skip_cmake'):
+            dist.skip_cmake = False
 
-    return display_only, dist.help_commands, dist.commands, dist.hide_listing
+    return (display_only, dist.help_commands, dist.commands,
+            dist.hide_listing, dist.force_cmake, dist.skip_cmake)
 
 
 def _check_skbuild_parameters(skbuild_kw):
@@ -216,7 +235,7 @@ def _package_data_contain_module(module, package_data):
     """
     (package, _, module_file) = module
     if package not in package_data:
-        return True
+        return False
     # We need to strip the package because a module entry
     # usually looks like this:
     #
@@ -226,12 +245,34 @@ def _package_data_contain_module(module, package_data):
     #
     #   {'foo.bar' : ['module.py']}
     if (strip_package(package.split("."), module_file)
-            not in package_data[package]):
+            in package_data[package]):
         return True
     return False
 
 
-# pylint:disable=too-many-locals
+def _should_run_cmake(commands, cmake_with_sdist):
+    """Return True if at least one command requiring ``cmake`` to run
+    is found in ``commands``."""
+    for expected_command in [
+        "build",
+        "develop",
+        "install",
+        "install_lib",
+        "bdist",
+        "bdist_dumb",
+        "bdist_egg"
+        "bdist_rpm",
+        "bdist_wininst",
+        "bdist_wheel",
+    ]:
+        if expected_command in commands:
+            return True
+    if "sdist" in commands and cmake_with_sdist:
+        return True
+    return False
+
+
+# pylint:disable=too-many-locals, too-many-branches
 def setup(*args, **kw):  # noqa: C901
     """This function wraps setup() so that we can run cmake, make,
     CMake build, then proceed as usual with setuptools, appending the
@@ -267,7 +308,8 @@ def setup(*args, **kw):  # noqa: C901
     parameters = {
         'cmake_args': [],
         'cmake_install_dir': '',
-        'cmake_source_dir': ''
+        'cmake_source_dir': '',
+        'cmake_with_sdist': False
     }
     skbuild_kw = {param: kw.pop(param, parameters[param])
                   for param in parameters}
@@ -291,13 +333,17 @@ def setup(*args, **kw):  # noqa: C901
         cmake_source_dir = os.path.relpath(cmake_source_dir)
 
     # Skip running CMake in the following cases:
+    # * flag "--skip-cmake" is provided
+    # * "display only" argument is provided (e.g  '--help', '--author', ...)
     # * no command-line arguments or invalid ones are provided
-    # * "display only" argument like '--help', '--help-commands'
-    #   or '--author' are provided
+    # * no command requiring cmake is provided
+    # * no CMakeLists.txt if found
     display_only = has_invalid_arguments = help_commands = False
+    force_cmake = skip_cmake = False
     commands = []
     try:
-        (display_only, help_commands, commands, hide_listing) = \
+        (display_only, help_commands, commands,
+         hide_listing, force_cmake, skip_cmake) = \
             _parse_setuptools_arguments(kw)
     except (DistutilsArgError, DistutilsGetoptError):
         has_invalid_arguments = True
@@ -307,13 +353,13 @@ def setup(*args, **kw):  # noqa: C901
     if not has_cmakelists:
         print('skipping skbuild (no CMakeLists.txt found)')
 
-    skip_cmake = (display_only
+    skip_cmake = (skip_cmake
+                  or display_only
                   or has_invalid_arguments
-                  or 'clean' in commands
-                  or 'egg_info' in commands
-                  or 'sdist' in commands
+                  or not _should_run_cmake(commands,
+                                           skbuild_kw["cmake_with_sdist"])
                   or not has_cmakelists)
-    if skip_cmake:
+    if skip_cmake and not force_cmake:
         if help_commands:
             # Prepend scikit-build help. Generate option descriptions using
             # argparse.
@@ -332,6 +378,8 @@ def setup(*args, **kw):  # noqa: C901
             print('')
         return upstream_setup(*args, **kw)
 
+    developer_mode = "develop" in commands
+
     packages = kw.get('packages', [])
     package_dir = kw.get('package_dir', {})
     package_data = kw.get('package_data', {}).copy()
@@ -354,10 +402,12 @@ def setup(*args, **kw):  # noqa: C901
 
     try:
         cmkr = cmaker.CMaker()
-        cmkr.configure(cmake_args,
-                       cmake_source_dir=cmake_source_dir,
-                       cmake_install_dir=skbuild_kw['cmake_install_dir'])
-        cmkr.make(make_args)
+        env = cmkr.configure(cmake_args,
+                             cmake_source_dir=cmake_source_dir,
+                             cmake_install_dir=skbuild_kw['cmake_install_dir'])
+        cmkr.make(make_args, env=env)
+    except SKBuildGeneratorNotFoundError as ex:
+        sys.exit(ex)
     except SKBuildError as ex:
         import traceback
         print("Traceback (most recent call last):")
@@ -378,8 +428,17 @@ def setup(*args, **kw):  # noqa: C901
                     data_files,
                     cmake_source_dir, skbuild_kw['cmake_install_dir'])
 
-    _consolidate(cmake_source_dir,
-                 packages, package_dir, py_modules, package_data, hide_listing)
+    if developer_mode:
+        for package, package_file_list in package_data.items():
+            for package_file in package_file_list:
+                package_file = os.path.join(package_dir[package], package_file)
+                cmake_file = os.path.join(CMAKE_INSTALL_DIR, package_file)
+                if os.path.exists(cmake_file):
+                    _copy_file(cmake_file, package_file, hide_listing)
+    else:
+        _consolidate(cmake_source_dir,
+                     packages, package_dir, py_modules, package_data,
+                     hide_listing)
 
     kw['package_data'] = package_data
     kw['package_dir'] = {
@@ -555,6 +614,27 @@ def _classify_files(install_paths, package_data, package_prefixes,
         del parent_dir, file_set
 
 
+def _copy_file(src_file, dest_file, hide_listing=True):
+    """Copy ``src_file`` to ``dest_file`` ensuring parent directory exists.
+
+    By default, message like `creating directory /path/to/package` and
+    `copying directory /src/path/to/package -> path/to/package` are displayed
+    on standard output. Setting ``hide_listing`` to False avoids message from
+    being displayed.
+    """
+    # Create directory if needed
+    dest_dir = os.path.dirname(dest_file)
+    if not os.path.exists(dest_dir):
+        if not hide_listing:
+            print("creating directory {}".format(dest_dir))
+        mkdir_p(dest_dir)
+
+    # Copy file
+    if not hide_listing:
+        print("copying {} -> {}".format(src_file, dest_file))
+    copyfile(src_file, dest_file)
+
+
 def _consolidate(
         cmake_source_dir, packages, package_dir, py_modules, package_data,
         hide_listing
@@ -598,25 +678,14 @@ def _consolidate(
     for entry in modules:
 
         # Check if module file should be copied into the CMake install tree.
-        if not _package_data_contain_module(entry, package_data):
+        if _package_data_contain_module(entry, package_data):
             continue
 
         (package, _, src_module_file) = entry
 
         # Copy missing module file
         dest_module_file = os.path.join(CMAKE_INSTALL_DIR, src_module_file)
-
-        # Create directory if needed
-        dest_module_dir = os.path.dirname(dest_module_file)
-        if not os.path.exists(dest_module_dir):
-            if not hide_listing:
-                print("creating directory {}".format(dest_module_dir))
-            mkdir_p(dest_module_dir)
-
-        # Copy file
-        if not hide_listing:
-            print("copying {} -> {}".format(src_module_file, dest_module_file))
-        copyfile(src_module_file, dest_module_file)
+        _copy_file(src_module_file, dest_module_file, hide_listing)
 
         # Since the mapping in package_data expects the package to be associated
         # with a list of files relative to the directory containing the package,
